'''
 sampleHandler.py
 
	Class handling sample routine with OASIS devices

  Copyright (c) 2024 Oliver Zobel - MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"),
  to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
  IN THE SOFTWARE.
  
 '''

import serial
import time
from scipy.io import savemat
import numpy as np
import matplotlib.pyplot as plt
from datetime import datetime

class sampleHandler():
    def __init__(self):
        
        self.OASISData = None
        self.OASISDataRaw = None
        self.t  = None
        
        # Dynamic Acquisition Parameters --------------------------------------------------
        self.Device = None
        self.t_sample = None
        self.f_sample = None
        self.VoltageRange = None
        self.Offset = None
        self.PRECACHE_SIZE = None
        self.sync_mode = None
        
        # Predefined Acquisition Parameters --------------------------------------------------
        self.CACHE_SIZE = 1500

        return
    
    def getAcquisitionParameters(self, Window, Device):
        
        # Acquisition Parameters --------------------------------------------------
        self.Device = Device
        self.serialSpeed = Device[3]
        self.t_sample = float(Window.lineEdit.text())
        self.f_sample = int(Window.lineEdit_2.text())
        self.VoltageRange = np.array([float(Window.comboBox_2.currentText()), float(Window.comboBox_3.currentText()), float(Window.comboBox_4.currentText()), float(Window.comboBox_5.currentText()), float(Window.comboBox_6.currentText()), float(Window.comboBox_7.currentText()), float(Window.comboBox_8.currentText()), float(Window.comboBox_9.currentText())])
        self.triggeredSample = Window.checkBox_3.isChecked()
        self.OversamplingID = Window.comboBox_10.currentIndex()
        self.fileName = Window.LastSampleDevice + "-" + datetime.now().strftime("%Y-%m-%d-%H.%M.%S")
        
        # Device depended parameters--------------------------------------------------
        self.BYTES_PER_SAMPLE = int(int(self.Device[1][2])) # Divide by 2 if 4 channels
        self.BYTES_PER_CACHE = self.BYTES_PER_SAMPLE * self.CACHE_SIZE
        
        # Triggered Sampling --------------------------------------------------
        if self.triggeredSample:
            self.V_TRIGG = float(Window.lineEdit_3.text())
            self.PRECACHE_SIZE = 1000
            self.Offset = 1
            
        else:
            self.V_TRIGG = 0
            self.PRECACHE_SIZE = 0
            self.Offset = 0

        # Sync mode
        if Window.checkBox_6.isChecked():
            if Window.radioButton.isChecked():
                self.sync_mode = 1
            else:
                self.sync_mode = 2
        else:
            self.sync_mode = 0

        # Preallocation
        self.OASISData = np.zeros([8, self.PRECACHE_SIZE + int(self.t_sample*self.f_sample)-self.Offset])

    def SampleSerial(self, printLogSignal, sampleAborted, sampleProgress):
        
        # Open serial connection
        if not self.connectSerial(printLogSignal,sampleAborted):
            self.OASISSerial.close()
            return

        # Set voltage ranges
        self.setVoltageRanges(printLogSignal)
        
        # Set oversampling factor
        self.setOversamplingFactor(printLogSignal)
 
        # Request new sample
        self.requestSample(printLogSignal)
        
        # Get data over serial interface
        if not self.retrieveDataSerial(printLogSignal,sampleAborted,sampleProgress):
            self.OASISSerial.close()
            return
            
        self.OASISSerial.close()
        
        # Convert Samples to voltages
        self.convertSamples()
        
    def resendSampleSerial(self, printLogSignal, sampleAborted, sampleProgress):
        
        # Open serial connection
        if not self.connectSerial(printLogSignal,sampleAborted):
            self.OASISSerial.close()
            return
 
        # Request previous sample
        printLogSignal.emit(f"[OASIS-GUI]: Requesting samples for filename '{self.fileName}' again.\n")
        self.requestPreviousSample(printLogSignal)
        
        # Get data over serial interface
        if not self.retrieveDataSerial(printLogSignal,sampleAborted,sampleProgress):
            self.OASISSerial.close()
            return
            
        self.OASISSerial.close()
        
        # Convert Samples to voltages
        self.convertSamples()

    # Serial Connection --------------------------------------------------
    def connectSerial(self, printLogSignal,sampleAborted): 
        try:
            self.OASISSerial = serial.Serial(port=self.Device[0], baudrate=self.serialSpeed, timeout=2)
        except (OSError, serial.SerialException):
            printLogSignal.emit("[OASIS-GUI]: DEVICE ERROR! Could not open serial communication.\n")
            printLogSignal.emit("[OASIS-GUI]: Data Acquisition aborted.\n")
            sampleAborted.emit()
            return False
        
        while self.OASISSerial.inWaiting():
            SerialAnswer = self.OASISSerial.readline()
            if SerialAnswer !=bytes("","utf-8"):
                if SerialAnswer.startswith(bytes("[OASIS]","utf-8")):
                    printLogSignal.emit(SerialAnswer.decode("utf-8",errors="ignore"))
                else:
                    printLogSignal.emit("[OASIS-GUI]: DEVICE ERROR! Unexpected serial communication content.\n")
                    printLogSignal.emit("[OASIS-GUI]: Data Acquisition aborted.\n")
                    sampleAborted.emit()
                    return False
        
        return True

    # Set Voltage Ranges --------------------------------------------------    
    def setVoltageRanges(self, printLogSignal): 
        
        VoltageRangeID = np.zeros(8,int)
        
        for k in range(0,8):
            if self.VoltageRange[k]==2.5:
                VoltageRangeID[k] = 1
            elif self.VoltageRange[k]==5:
                VoltageRangeID[k] = 2
            elif self.VoltageRange[k]==6.25:
                VoltageRangeID[k] = 3
            elif self.VoltageRange[k]==10:
                VoltageRangeID[k] = 4
            elif self.VoltageRange[k]==12.5:
                VoltageRangeID[k] = 5
            else:
                raise ValueError("Voltage Range " + str(self.VoltageRange[k]) + " for channel " + str(k+1) + " is invalid.")

        self.OASISSerial.write(bytes("OASIS.SetVoltageRange(" + str(VoltageRangeID[0]) + "," + str(VoltageRangeID[1]) + "," + str(VoltageRangeID[2]) + ","  + 
                str(VoltageRangeID[3]) + "," + str(VoltageRangeID[4]) + "," + str(VoltageRangeID[5]) + "," + str(VoltageRangeID[6]) + ","  + 
                str(VoltageRangeID[7]) + ")","utf-8"))

        then = time.time()
        while True:
            if time.time()-then>2: # Resend command if stuck
                then = time.time()
                self.OASISSerial.write(bytes("OASIS.SetVoltageRange(" + str(VoltageRangeID[0]) + "," + str(VoltageRangeID[1]) + "," + str(VoltageRangeID[2]) + ","  + 
                        str(VoltageRangeID[3]) + "," + str(VoltageRangeID[4]) + "," + str(VoltageRangeID[5]) + "," + str(VoltageRangeID[6]) + ","  + 
                        str(VoltageRangeID[7]) + ")","utf-8"))
            if self.OASISSerial.inWaiting():
                SerialAnswer = self.OASISSerial.readline()
                if SerialAnswer!=bytes("[OASIS] Voltage ranges set.\r\n","utf-8"):
                    if SerialAnswer !=bytes("","utf-8"):
                        printLogSignal.emit(SerialAnswer.decode("utf-8",errors="ignore"))
                else:
                    break
    # Set oversampling factor --------------------------------------------------
    def setOversamplingFactor(self, printLogSignal):
        self.OASISSerial.write(bytes("OASIS.SetOversampling(" + str(self.OversamplingID) + ")","utf-8"))
        
        then = time.time()
        while True:
            if time.time()-then>2: # Resend command if stuck
                then = time.time()
                self.OASISSerial.write(bytes("OASIS.SetOversampling(" + str(self.OversamplingID) + ")","utf-8"))
            if self.OASISSerial.inWaiting():
                SerialAnswer = self.OASISSerial.readline()
                if SerialAnswer!=bytes("[OASIS] Oversampling factor set.\r\n","utf-8"):
                    if SerialAnswer !=bytes("","utf-8"):
                        printLogSignal.emit(SerialAnswer.decode("utf-8",errors="ignore"))
                else:
                    break
    # Write OASIS.Sample() command  --------------------------------------------------
    def requestSample(self, printLogSignal):
        self.OASISSerial.write(bytes("OASIS.Sample(" + str(self.t_sample) + "," + str(self.f_sample) + "," + str(self.V_TRIGG) + "," + str(self.sync_mode) + "," + str(self.fileName) + ")","utf-8"))
        
        then = time.time()
        while True:
            if time.time()-then>2: # Resend command if stuck
                then = time.time()
                self.OASISSerial.write(bytes("OASIS.Sample(" + str(self.t_sample) + "," + str(self.f_sample) + "," + str(self.V_TRIGG) + "," + str(self.sync_mode) + "," + str(self.fileName) + ")","utf-8"))
            if self.OASISSerial.inWaiting():
                SerialAnswer = self.OASISSerial.readline()
                if SerialAnswer!=bytes("[OASIS] Starting sample procedure...\r\n","utf-8"):
                    if SerialAnswer !=bytes("","utf-8"):
                        printLogSignal.emit(SerialAnswer.decode("utf-8",errors="ignore"))
                else:
                    break
                
    # Write OASIS.Sample() command  --------------------------------------------------
    def requestPreviousSample(self, printLogSignal):
        self.OASISSerial.write(bytes("OASIS.SendData(" + str(self.fileName) + ")","utf-8"))
        
        then = time.time()
        while True:
            if time.time()-then>2: # Resend command if stuck
                then = time.time()
                self.OASISSerial.write(bytes("OASIS.SendData(" + str(self.fileName) + ")","utf-8"))
            if self.OASISSerial.inWaiting():
                SerialAnswer = self.OASISSerial.readline()
                if SerialAnswer!=bytes("[OASIS] Sending sample files named " + str(self.fileName) + "...\r\n","utf-8"):
                    if SerialAnswer !=bytes("","utf-8"):
                        printLogSignal.emit(SerialAnswer.decode("utf-8",errors="ignore"))
                else:
                    break
                
    # Retrieve samples via serial interface  --------------------------------------------------
    def retrieveDataSerial(self,printLogSignal,sampleAborted,sampleProgress):
        
        # Variables
        OASISRcvBuffer = np.zeros([int(self.t_sample*self.f_sample) * self.BYTES_PER_SAMPLE - self.Offset*self.BYTES_PER_SAMPLE])
        OASISDataRawMain = np.zeros([int(self.t_sample*self.f_sample) - self.Offset, self.BYTES_PER_SAMPLE])
        OASISDataRawPreTrigg = np.zeros([self.PRECACHE_SIZE, self.BYTES_PER_SAMPLE])
        
        # Wait for incoming data --------------------------------------------------
        while True:
            if self.OASISSerial.inWaiting():
                SerialAnswer = self.OASISSerial.readline()
                if SerialAnswer!=bytes("<>\r\n","utf-8"):
                    if SerialAnswer==bytes("[OASIS] WiFi is ON. Disabling WiFi for Data Acquisition over Serial...\r\n","utf-8"):
                        then = time.time()
                        while time.time()-then<4:
                            pass
                        
                    if SerialAnswer !=bytes("","utf-8"):
                        printLogSignal.emit(SerialAnswer.decode("utf-8",errors="ignore"))
                    if SerialAnswer==bytes("[OASIS] Sampling aborted.\r\n","utf-8") or SerialAnswer==bytes("[OASIS] FATAL ERROR - Sampling too fast! Data processing did not finish in time.\r\n","utf-8"):
                        printLogSignal.emit("[OASIS-GUI]: Data Acquisition aborted.\n")
                        sampleAborted.emit()
                        return False
                else:
                    break
        
        dataRcv = 0
        
        # Read buffer and sort in array --------------------------------------------------
        while dataRcv != (int(self.t_sample*self.f_sample/self.CACHE_SIZE)*self.BYTES_PER_CACHE):
            if(self.OASISSerial.inWaiting()):
                _OASISRcvBuffer = self.OASISSerial.read(self.BYTES_PER_CACHE)
                
                if(len(_OASISRcvBuffer)!=self.BYTES_PER_CACHE):
                    printLogSignal.emit("[OASIS-GUI]: DEVICE ERROR! Unexpected serial communication timeout.\n")
                    printLogSignal.emit("[OASIS-GUI]: Data Acquisition aborted.\n")
                    sampleAborted.emit()
                    return False
                    
                for i, _byte in enumerate(_OASISRcvBuffer):
                    OASISRcvBuffer[i+dataRcv]=_byte
                    tmp = i
            
                dataRcv += tmp + 1
                sampleProgress.emit(int(dataRcv/(int(self.t_sample*self.f_sample/self.CACHE_SIZE)*self.BYTES_PER_CACHE)*100))
                
        _OASISRcvBuffer = self.OASISSerial.read(int(self.t_sample*self.f_sample*self.BYTES_PER_SAMPLE)-int(self.t_sample*self.f_sample/self.CACHE_SIZE)*self.BYTES_PER_CACHE - self.BYTES_PER_SAMPLE*self.Offset)
    
        for i, _byte in enumerate(_OASISRcvBuffer):
            OASISRcvBuffer[i+dataRcv]=_byte

        if(len(OASISRcvBuffer)==(int(self.t_sample*self.f_sample) * self.BYTES_PER_SAMPLE - self.Offset*self.BYTES_PER_SAMPLE)):
            sampleProgress.emit(100)
        else:
            printLogSignal.emit("[OASIS-GUI]: DEVICE ERROR! Data has been lost during transmission.\n")
            printLogSignal.emit("[OASIS-GUI]: Data Acquisition aborted.\n")
            sampleAborted.emit()
            return False

        
        for i, _byte in enumerate(OASISRcvBuffer):
            OASISDataRawMain[int(i/self.BYTES_PER_SAMPLE),i%self.BYTES_PER_SAMPLE]=_byte
        
        # Convert to Integer --------------------------------------------------
        OASISDataRawMain = OASISDataRawMain.astype(int)
        
        # If Triggered sampling acquire Pre-Trigg Data; assemble everything into one array --------------------------------------------------
        if self.triggeredSample:
        
            # Discard offset sample
            self.OASISSerial.read(self.BYTES_PER_SAMPLE*self.Offset)
            
            # Wait for OASIS to be ready to send pre-trigger data
            time.sleep(0.5)
            
            # Retrieve Pre-Trigger Data
            self.OASISSerial.write(bytes("Drq()","utf-8"))
            
            # Wait for incoming data
            then = time.time()
            while True:
                if time.time()-then>1: # Resend command if stuck
                    then = time.time()
                    self.OASISSerial.write(bytes("Drq()","utf-8"))
                if self.OASISSerial.inWaiting():
                    SerialAnswer = self.OASISSerial.readline()
                    if SerialAnswer!=bytes("<>\r\n","utf-8"):
                        if SerialAnswer !=bytes("","utf-8"):
                            printLogSignal.emit(SerialAnswer.decode("utf-8",errors="ignore"))
                    else:
                        break

            # Read buffer and sort in array
            _OASISDataRawPreTrigg = self.OASISSerial.read(self.BYTES_PER_SAMPLE*self.PRECACHE_SIZE)
            
            for i, _byte in enumerate(_OASISDataRawPreTrigg):
                OASISDataRawPreTrigg[int(i/self.BYTES_PER_SAMPLE),i%self.BYTES_PER_SAMPLE]=_byte
                
            # Convert to Integer
            OASISDataRawPreTrigg = OASISDataRawPreTrigg.astype(int)
            
            self.OASISDataRaw = np.concatenate((OASISDataRawPreTrigg,OASISDataRawMain))
            
        else:
            self.OASISDataRaw = OASISDataRawMain
            
        printLogSignal.emit("[OASIS-GUI] Data Acquisition finished.\n\n")
        
        return True
    
    # Convert bytes to voltages  --------------------------------------------------
    def convertSamples(self):
        
        # Variables --------------------------------------------------
        OASISChannelData = np.zeros([self.PRECACHE_SIZE + int(self.t_sample*self.f_sample) - self.Offset, 8])
        
        # Seperation of channel bits --------------------------------------------------
        if(self.BYTES_PER_SAMPLE==18):
            for k in range(0,len(self.OASISDataRaw)):
                OASISChannelData[k,0] = (self.OASISDataRaw[k,0] << 10) + (self.OASISDataRaw[k,1] << 2) + (self.OASISDataRaw[k,2] >> 6)
                OASISChannelData[k,1] = ((self.OASISDataRaw[k,2]-((self.OASISDataRaw[k,2] >> 6) << 6)) << 12) + (self.OASISDataRaw[k,3] << 4) + (self.OASISDataRaw[k,4] >> 4)
                OASISChannelData[k,2] = ((self.OASISDataRaw[k,4]-((self.OASISDataRaw[k,4] >> 4) << 4)) << 14) + (self.OASISDataRaw[k,5] << 6) + (self.OASISDataRaw[k,6] >> 2)
                OASISChannelData[k,3] = ((self.OASISDataRaw[k,6]-((self.OASISDataRaw[k,6] >> 2) << 2)) << 16) + (self.OASISDataRaw[k,7] << 8) + (self.OASISDataRaw[k,8])
                OASISChannelData[k,4] = (self.OASISDataRaw[k,9] << 10) + (self.OASISDataRaw[k,10] << 2) + (self.OASISDataRaw[k,11] >> 6)
                OASISChannelData[k,5] = ((self.OASISDataRaw[k,11]-((self.OASISDataRaw[k,11] >> 6) << 6)) << 12) + (self.OASISDataRaw[k,12] << 4) + (self.OASISDataRaw[k,13] >> 4)
                OASISChannelData[k,6] = ((self.OASISDataRaw[k,13]-((self.OASISDataRaw[k,13] >> 4) << 4)) << 14) + (self.OASISDataRaw[k,14] << 6) + (self.OASISDataRaw[k,15] >> 2)
                OASISChannelData[k,7] = ((self.OASISDataRaw[k,15]-((self.OASISDataRaw[k,15] >> 2) << 2)) << 16) + (self.OASISDataRaw[k,16] << 8) + (self.OASISDataRaw[k,17])
                
        elif(self.BYTES_PER_SAMPLE==8):
            for k in range(0,len(self.OASISDataRaw)):
                OASISChannelData[k,0] = (self.OASISDataRaw[k,0] << 8) + (self.OASISDataRaw[k,1])
                OASISChannelData[k,1] = (self.OASISDataRaw[k,2] << 8) + (self.OASISDataRaw[k,3])
                OASISChannelData[k,2] = (self.OASISDataRaw[k,4] << 8) + (self.OASISDataRaw[k,5])
                OASISChannelData[k,3] = (self.OASISDataRaw[k,6] << 8) + (self.OASISDataRaw[k,7])
                
        # Convert to Voltage --------------------------------------------------
        BitDivider = 2**int(self.Device[1][2])/2
        
        for k in range(0,len(self.OASISDataRaw)):
            for i in range(0,8):
                if OASISChannelData[k,i]/BitDivider <= 1:
                    self.OASISData[i,k] = (OASISChannelData[k,i]*self.VoltageRange[i])/BitDivider;
                else:
                    self.OASISData[i,k] = ((OASISChannelData[k,i]-2*BitDivider)/BitDivider)*self.VoltageRange[i];
        
        # Assemble time vector --------------------------------------------------
        if self.triggeredSample:
            N = np.arange((1-self.PRECACHE_SIZE), self.t_sample*self.f_sample, 1)
            self.t = N/self.f_sample
        else:
            self.t = np.arange(0, self.t_sample, 1/self.f_sample)

    def plotData(self):
        # Data plot --------------------------------------------------
        fig, axs = plt.subplots(2, 4, num=self.fileName)
        j = 0
        axs[0, 0].set_ylabel("Voltage / V")
        axs[1, 0].set_ylabel("Voltage / V")
        for k in range(0,8):
            if k==4:
                j = 1
            fig.set_size_inches(18, 10.5, forward=True)
            axs[j, k-j*4].plot(self.t,self.OASISData[k])
            axs[j, k-j*4].set_title("Channel" + str(k+1))
            axs[j, k-j*4].set_xlabel("Time / s")
            
        if self.triggeredSample:
            axs[0,0].axvline(0,color='black',linestyle='--')
        plt.tight_layout()
        plt.show()

    def saveData(self, Window):
        # Save Data --------------------------------------------------
        saveName = self.fileName + ".mat"

        Window.textEdit.append(f"[OASIS-GUI]: Saving acquired data as {saveName}\n")

        try:
            savemat(saveName, {'OASISChannel': self.OASISData, 'OASISTime': self.t})
            Window.textEdit.append(f"[OASIS-GUI]: {saveName} successfully saved.\n")
        except PermissionError:
            Window.textEdit.append("[OASIS-GUI]: ERROR! Can not write to filesystem. Are you running as admin?\n")
            Window.progressBar_2.setVisible(True)
        
        Window.textEdit.ensureCursorVisible()
        
        return saveName


# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'ui/OASISGui.ui'
#
# Created by: PyQt5 UI code generator 5.15.9
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(1040, 830)
        MainWindow.setMinimumSize(QtCore.QSize(1040, 830))
        MainWindow.setMaximumSize(QtCore.QSize(1040, 830))
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.tabWidget = QtWidgets.QTabWidget(self.centralwidget)
        self.tabWidget.setEnabled(False)
        self.tabWidget.setGeometry(QtCore.QRect(0, 320, 1050, 511))
        self.tabWidget.setObjectName("tabWidget")
        self.tab = QtWidgets.QWidget()
        self.tab.setObjectName("tab")
        self.groupBox = QtWidgets.QGroupBox(self.tab)
        self.groupBox.setGeometry(QtCore.QRect(10, 10, 351, 401))
        self.groupBox.setObjectName("groupBox")
        self.label_12 = QtWidgets.QLabel(self.groupBox)
        self.label_12.setGeometry(QtCore.QRect(10, 30, 181, 16))
        self.label_12.setObjectName("label_12")
        self.label_13 = QtWidgets.QLabel(self.groupBox)
        self.label_13.setGeometry(QtCore.QRect(10, 60, 181, 16))
        self.label_13.setObjectName("label_13")
        self.lineEdit = QtWidgets.QLineEdit(self.groupBox)
        self.lineEdit.setGeometry(QtCore.QRect(200, 30, 131, 20))
        self.lineEdit.setObjectName("lineEdit")
        self.lineEdit_2 = QtWidgets.QLineEdit(self.groupBox)
        self.lineEdit_2.setGeometry(QtCore.QRect(200, 60, 131, 20))
        self.lineEdit_2.setObjectName("lineEdit_2")
        self.groupBox_2 = QtWidgets.QGroupBox(self.groupBox)
        self.groupBox_2.setGeometry(QtCore.QRect(10, 300, 331, 91))
        self.groupBox_2.setObjectName("groupBox_2")
        self.checkBox_3 = QtWidgets.QCheckBox(self.groupBox_2)
        self.checkBox_3.setGeometry(QtCore.QRect(10, 30, 231, 18))
        self.checkBox_3.setObjectName("checkBox_3")
        self.label_18 = QtWidgets.QLabel(self.groupBox_2)
        self.label_18.setEnabled(False)
        self.label_18.setGeometry(QtCore.QRect(10, 60, 161, 16))
        self.label_18.setObjectName("label_18")
        self.lineEdit_3 = QtWidgets.QLineEdit(self.groupBox_2)
        self.lineEdit_3.setEnabled(False)
        self.lineEdit_3.setGeometry(QtCore.QRect(190, 60, 131, 20))
        self.lineEdit_3.setObjectName("lineEdit_3")
        self.groupBox_11 = QtWidgets.QGroupBox(self.groupBox)
        self.groupBox_11.setGeometry(QtCore.QRect(10, 120, 331, 171))
        self.groupBox_11.setObjectName("groupBox_11")
        self.comboBox_9 = QtWidgets.QComboBox(self.groupBox_11)
        self.comboBox_9.setEnabled(False)
        self.comboBox_9.setGeometry(QtCore.QRect(250, 140, 71, 22))
        self.comboBox_9.setObjectName("comboBox_9")
        self.label_24 = QtWidgets.QLabel(self.groupBox_11)
        self.label_24.setGeometry(QtCore.QRect(180, 110, 61, 16))
        self.label_24.setObjectName("label_24")
        self.comboBox_6 = QtWidgets.QComboBox(self.groupBox_11)
        self.comboBox_6.setEnabled(False)
        self.comboBox_6.setGeometry(QtCore.QRect(250, 50, 71, 22))
        self.comboBox_6.setObjectName("comboBox_6")
        self.comboBox_2 = QtWidgets.QComboBox(self.groupBox_11)
        self.comboBox_2.setEnabled(False)
        self.comboBox_2.setGeometry(QtCore.QRect(90, 50, 71, 22))
        self.comboBox_2.setObjectName("comboBox_2")
        self.label_23 = QtWidgets.QLabel(self.groupBox_11)
        self.label_23.setGeometry(QtCore.QRect(180, 80, 61, 16))
        self.label_23.setObjectName("label_23")
        self.label_17 = QtWidgets.QLabel(self.groupBox_11)
        self.label_17.setGeometry(QtCore.QRect(20, 140, 61, 16))
        self.label_17.setObjectName("label_17")
        self.label_25 = QtWidgets.QLabel(self.groupBox_11)
        self.label_25.setGeometry(QtCore.QRect(180, 50, 61, 16))
        self.label_25.setObjectName("label_25")
        self.label_14 = QtWidgets.QLabel(self.groupBox_11)
        self.label_14.setGeometry(QtCore.QRect(20, 50, 61, 16))
        self.label_14.setObjectName("label_14")
        self.comboBox_4 = QtWidgets.QComboBox(self.groupBox_11)
        self.comboBox_4.setEnabled(False)
        self.comboBox_4.setGeometry(QtCore.QRect(90, 110, 71, 22))
        self.comboBox_4.setObjectName("comboBox_4")
        self.comboBox_7 = QtWidgets.QComboBox(self.groupBox_11)
        self.comboBox_7.setEnabled(False)
        self.comboBox_7.setGeometry(QtCore.QRect(250, 80, 71, 22))
        self.comboBox_7.setObjectName("comboBox_7")
        self.label_16 = QtWidgets.QLabel(self.groupBox_11)
        self.label_16.setGeometry(QtCore.QRect(20, 110, 61, 16))
        self.label_16.setObjectName("label_16")
        self.label_15 = QtWidgets.QLabel(self.groupBox_11)
        self.label_15.setGeometry(QtCore.QRect(20, 80, 61, 16))
        self.label_15.setObjectName("label_15")
        self.comboBox_3 = QtWidgets.QComboBox(self.groupBox_11)
        self.comboBox_3.setEnabled(False)
        self.comboBox_3.setGeometry(QtCore.QRect(90, 80, 71, 22))
        self.comboBox_3.setObjectName("comboBox_3")
        self.comboBox_5 = QtWidgets.QComboBox(self.groupBox_11)
        self.comboBox_5.setEnabled(False)
        self.comboBox_5.setGeometry(QtCore.QRect(90, 140, 71, 22))
        self.comboBox_5.setObjectName("comboBox_5")
        self.comboBox_8 = QtWidgets.QComboBox(self.groupBox_11)
        self.comboBox_8.setEnabled(False)
        self.comboBox_8.setGeometry(QtCore.QRect(250, 110, 71, 22))
        self.comboBox_8.setObjectName("comboBox_8")
        self.label_26 = QtWidgets.QLabel(self.groupBox_11)
        self.label_26.setGeometry(QtCore.QRect(180, 140, 61, 16))
        self.label_26.setObjectName("label_26")
        self.label_30 = QtWidgets.QLabel(self.groupBox_11)
        self.label_30.setGeometry(QtCore.QRect(20, 22, 151, 16))
        self.label_30.setObjectName("label_30")
        self.comboBox_11 = QtWidgets.QComboBox(self.groupBox_11)
        self.comboBox_11.setEnabled(False)
        self.comboBox_11.setGeometry(QtCore.QRect(190, 20, 131, 22))
        self.comboBox_11.setObjectName("comboBox_11")
        self.label_29 = QtWidgets.QLabel(self.groupBox)
        self.label_29.setGeometry(QtCore.QRect(10, 90, 181, 16))
        self.label_29.setObjectName("label_29")
        self.comboBox_10 = QtWidgets.QComboBox(self.groupBox)
        self.comboBox_10.setGeometry(QtCore.QRect(200, 90, 131, 22))
        self.comboBox_10.setObjectName("comboBox_10")
        self.groupBox_3 = QtWidgets.QGroupBox(self.tab)
        self.groupBox_3.setGeometry(QtCore.QRect(380, 310, 421, 101))
        self.groupBox_3.setObjectName("groupBox_3")
        self.checkBox_4 = QtWidgets.QCheckBox(self.groupBox_3)
        self.checkBox_4.setGeometry(QtCore.QRect(20, 30, 261, 18))
        self.checkBox_4.setChecked(True)
        self.checkBox_4.setObjectName("checkBox_4")
        self.checkBox_5 = QtWidgets.QCheckBox(self.groupBox_3)
        self.checkBox_5.setGeometry(QtCore.QRect(20, 60, 261, 18))
        self.checkBox_5.setObjectName("checkBox_5")
        self.pushButton_3 = QtWidgets.QPushButton(self.groupBox_3)
        self.pushButton_3.setEnabled(False)
        self.pushButton_3.setGeometry(QtCore.QRect(260, 30, 151, 22))
        self.pushButton_3.setObjectName("pushButton_3")
        self.pushButton_4 = QtWidgets.QPushButton(self.groupBox_3)
        self.pushButton_4.setEnabled(False)
        self.pushButton_4.setGeometry(QtCore.QRect(260, 60, 151, 22))
        self.pushButton_4.setObjectName("pushButton_4")
        self.pushButton_5 = QtWidgets.QPushButton(self.groupBox_3)
        self.pushButton_5.setEnabled(False)
        self.pushButton_5.setGeometry(QtCore.QRect(490, 20, 151, 22))
        self.pushButton_5.setObjectName("pushButton_5")
        self.groupBox_4 = QtWidgets.QGroupBox(self.tab)
        self.groupBox_4.setGeometry(QtCore.QRect(380, 10, 651, 291))
        self.groupBox_4.setObjectName("groupBox_4")
        self.textEdit = QtWidgets.QTextEdit(self.groupBox_4)
        self.textEdit.setGeometry(QtCore.QRect(10, 20, 631, 261))
        self.textEdit.setReadOnly(True)
        self.textEdit.setObjectName("textEdit")
        self.progressBar = QtWidgets.QProgressBar(self.tab)
        self.progressBar.setEnabled(False)
        self.progressBar.setGeometry(QtCore.QRect(10, 420, 1021, 31))
        self.progressBar.setAcceptDrops(False)
        self.progressBar.setStyleSheet("")
        self.progressBar.setMaximum(100)
        self.progressBar.setProperty("value", 0)
        self.progressBar.setTextVisible(True)
        self.progressBar.setOrientation(QtCore.Qt.Horizontal)
        self.progressBar.setTextDirection(QtWidgets.QProgressBar.TopToBottom)
        self.progressBar.setObjectName("progressBar")
        self.progressBar_2 = QtWidgets.QProgressBar(self.tab)
        self.progressBar_2.setEnabled(False)
        self.progressBar_2.setGeometry(QtCore.QRect(10, 420, 981, 31))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.progressBar_2.sizePolicy().hasHeightForWidth())
        self.progressBar_2.setSizePolicy(sizePolicy)
        self.progressBar_2.setAcceptDrops(False)
        self.progressBar_2.setAutoFillBackground(False)
        self.progressBar_2.setStyleSheet("QProgressBar::chunk{background-color: red;}")
        self.progressBar_2.setMaximum(100)
        self.progressBar_2.setProperty("value", 100)
        self.progressBar_2.setTextVisible(False)
        self.progressBar_2.setObjectName("progressBar_2")
        self.groupBox_7 = QtWidgets.QGroupBox(self.tab)
        self.groupBox_7.setGeometry(QtCore.QRect(820, 310, 211, 101))
        self.groupBox_7.setObjectName("groupBox_7")
        self.pushButton_2 = QtWidgets.QPushButton(self.groupBox_7)
        self.pushButton_2.setEnabled(False)
        self.pushButton_2.setGeometry(QtCore.QRect(10, 60, 191, 30))
        font = QtGui.QFont()
        font.setPointSize(9)
        self.pushButton_2.setFont(font)
        self.pushButton_2.setObjectName("pushButton_2")
        self.pushButton_7 = QtWidgets.QPushButton(self.groupBox_7)
        self.pushButton_7.setEnabled(False)
        self.pushButton_7.setGeometry(QtCore.QRect(10, 20, 191, 30))
        font = QtGui.QFont()
        font.setPointSize(9)
        self.pushButton_7.setFont(font)
        self.pushButton_7.setObjectName("pushButton_7")
        self.groupBox_4.raise_()
        self.groupBox_3.raise_()
        self.groupBox.raise_()
        self.progressBar.raise_()
        self.progressBar_2.raise_()
        self.groupBox_7.raise_()
        self.tabWidget.addTab(self.tab, "")
        self.label = QtWidgets.QLabel(self.centralwidget)
        self.label.setGeometry(QtCore.QRect(0, 20, 1041, 41))
        font = QtGui.QFont()
        font.setPointSize(20)
        self.label.setFont(font)
        self.label.setCursor(QtGui.QCursor(QtCore.Qt.ArrowCursor))
        self.label.setAlignment(QtCore.Qt.AlignCenter)
        self.label.setTextInteractionFlags(QtCore.Qt.NoTextInteraction)
        self.label.setObjectName("label")
        self.comboBox = QtWidgets.QComboBox(self.centralwidget)
        self.comboBox.setEnabled(False)
        self.comboBox.setGeometry(QtCore.QRect(310, 80, 411, 22))
        self.comboBox.setObjectName("comboBox")
        self.comboBox.addItem("")
        self.pushButton = QtWidgets.QPushButton(self.centralwidget)
        self.pushButton.setEnabled(True)
        self.pushButton.setGeometry(QtCore.QRect(740, 80, 121, 23))
        self.pushButton.setObjectName("pushButton")
        self.label_19 = QtWidgets.QLabel(self.centralwidget)
        self.label_19.setGeometry(QtCore.QRect(870, 70, 42, 42))
        self.label_19.setText("")
        self.label_19.setPixmap(QtGui.QPixmap(":/Misc/resources/spin.gif"))
        self.label_19.setScaledContents(True)
        self.label_19.setObjectName("label_19")
        self.groupBox_5 = QtWidgets.QGroupBox(self.centralwidget)
        self.groupBox_5.setEnabled(False)
        self.groupBox_5.setGeometry(QtCore.QRect(720, 140, 281, 171))
        self.groupBox_5.setObjectName("groupBox_5")
        self.checkBox_6 = QtWidgets.QCheckBox(self.groupBox_5)
        self.checkBox_6.setGeometry(QtCore.QRect(20, 30, 221, 21))
        self.checkBox_6.setObjectName("checkBox_6")
        self.radioButton = QtWidgets.QRadioButton(self.groupBox_5)
        self.radioButton.setEnabled(False)
        self.radioButton.setGeometry(QtCore.QRect(40, 100, 221, 20))
        self.radioButton.setChecked(True)
        self.radioButton.setObjectName("radioButton")
        self.radioButton_2 = QtWidgets.QRadioButton(self.groupBox_5)
        self.radioButton_2.setEnabled(False)
        self.radioButton_2.setGeometry(QtCore.QRect(40, 130, 221, 20))
        self.radioButton_2.setObjectName("radioButton_2")
        self.label_20 = QtWidgets.QLabel(self.groupBox_5)
        self.label_20.setEnabled(False)
        self.label_20.setGeometry(QtCore.QRect(20, 70, 131, 16))
        self.label_20.setObjectName("label_20")
        self.label_21 = QtWidgets.QLabel(self.groupBox_5)
        self.label_21.setEnabled(False)
        self.label_21.setGeometry(QtCore.QRect(220, 110, 41, 41))
        self.label_21.setText("")
        self.label_21.setPixmap(QtGui.QPixmap(":/Misc/resources/unlock.png"))
        self.label_21.setScaledContents(True)
        self.label_21.setObjectName("label_21")
        self.label_22 = QtWidgets.QLabel(self.centralwidget)
        self.label_22.setGeometry(QtCore.QRect(90, 5, 141, 131))
        self.label_22.setText("")
        self.label_22.setPixmap(QtGui.QPixmap("ui/resources/icons/128x128.png"))
        self.label_22.setObjectName("label_22")
        self.groupBox_6 = QtWidgets.QGroupBox(self.centralwidget)
        self.groupBox_6.setGeometry(QtCore.QRect(40, 140, 651, 171))
        self.groupBox_6.setObjectName("groupBox_6")
        self.label_28 = QtWidgets.QLabel(self.groupBox_6)
        self.label_28.setGeometry(QtCore.QRect(330, 20, 200, 14))
        self.label_28.setText("")
        self.label_28.setObjectName("label_28")
        self.checkBox = QtWidgets.QCheckBox(self.groupBox_6)
        self.checkBox.setEnabled(False)
        self.checkBox.setGeometry(QtCore.QRect(330, 120, 131, 18))
        self.checkBox.setObjectName("checkBox")
        self.label_11 = QtWidgets.QLabel(self.groupBox_6)
        self.label_11.setGeometry(QtCore.QRect(330, 100, 200, 14))
        self.label_11.setText("")
        self.label_11.setObjectName("label_11")
        self.label_6 = QtWidgets.QLabel(self.groupBox_6)
        self.label_6.setGeometry(QtCore.QRect(190, 100, 171, 16))
        self.label_6.setObjectName("label_6")
        self.label_9 = QtWidgets.QLabel(self.groupBox_6)
        self.label_9.setGeometry(QtCore.QRect(330, 60, 200, 14))
        self.label_9.setText("")
        self.label_9.setObjectName("label_9")
        self.label_27 = QtWidgets.QLabel(self.groupBox_6)
        self.label_27.setGeometry(QtCore.QRect(190, 20, 171, 16))
        self.label_27.setObjectName("label_27")
        self.label_4 = QtWidgets.QLabel(self.groupBox_6)
        self.label_4.setGeometry(QtCore.QRect(190, 60, 171, 16))
        self.label_4.setObjectName("label_4")
        self.checkBox_2 = QtWidgets.QCheckBox(self.groupBox_6)
        self.checkBox_2.setEnabled(False)
        self.checkBox_2.setGeometry(QtCore.QRect(330, 140, 251, 18))
        self.checkBox_2.setObjectName("checkBox_2")
        self.label_8 = QtWidgets.QLabel(self.groupBox_6)
        self.label_8.setGeometry(QtCore.QRect(330, 40, 200, 14))
        self.label_8.setText("")
        self.label_8.setObjectName("label_8")
        self.label_10 = QtWidgets.QLabel(self.groupBox_6)
        self.label_10.setGeometry(QtCore.QRect(330, 80, 200, 14))
        self.label_10.setText("")
        self.label_10.setObjectName("label_10")
        self.label_2 = QtWidgets.QLabel(self.groupBox_6)
        self.label_2.setGeometry(QtCore.QRect(40, 30, 111, 131))
        self.label_2.setText("")
        self.label_2.setPixmap(QtGui.QPixmap(":/Boards/resources/boards/Unknown.png"))
        self.label_2.setScaledContents(False)
        self.label_2.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
        self.label_2.setObjectName("label_2")
        self.label_7 = QtWidgets.QLabel(self.groupBox_6)
        self.label_7.setGeometry(QtCore.QRect(190, 120, 171, 16))
        self.label_7.setObjectName("label_7")
        self.label_3 = QtWidgets.QLabel(self.groupBox_6)
        self.label_3.setGeometry(QtCore.QRect(190, 40, 171, 16))
        self.label_3.setObjectName("label_3")
        self.label_5 = QtWidgets.QLabel(self.groupBox_6)
        self.label_5.setGeometry(QtCore.QRect(190, 80, 171, 16))
        self.label_5.setObjectName("label_5")
        MainWindow.setCentralWidget(self.centralwidget)
        self.menuBar = QtWidgets.QMenuBar(MainWindow)
        self.menuBar.setGeometry(QtCore.QRect(0, 0, 1040, 26))
        self.menuBar.setObjectName("menuBar")
        self.menuAbout = QtWidgets.QMenu(self.menuBar)
        self.menuAbout.setContextMenuPolicy(QtCore.Qt.NoContextMenu)
        self.menuAbout.setObjectName("menuAbout")
        self.menuFile = QtWidgets.QMenu(self.menuBar)
        self.menuFile.setObjectName("menuFile")
        MainWindow.setMenuBar(self.menuBar)
        self.actionAbout = QtWidgets.QAction(MainWindow)
        self.actionAbout.setObjectName("actionAbout")
        self.actionSearch_Devices = QtWidgets.QAction(MainWindow)
        self.actionSearch_Devices.setObjectName("actionSearch_Devices")
        self.actionSerial_Sample = QtWidgets.QAction(MainWindow)
        self.actionSerial_Sample.setObjectName("actionSerial_Sample")
        self.actionDevice_Selected_Changed = QtWidgets.QAction(MainWindow)
        self.actionDevice_Selected_Changed.setObjectName("actionDevice_Selected_Changed")
        self.actionRange_Channel1_Changed = QtWidgets.QAction(MainWindow)
        self.actionRange_Channel1_Changed.setObjectName("actionRange_Channel1_Changed")
        self.actionshow_Previous_Data = QtWidgets.QAction(MainWindow)
        self.actionshow_Previous_Data.setObjectName("actionshow_Previous_Data")
        self.actionsave_Previous_Data = QtWidgets.QAction(MainWindow)
        self.actionsave_Previous_Data.setObjectName("actionsave_Previous_Data")
        self.actionLockDevice = QtWidgets.QAction(MainWindow)
        self.actionLockDevice.setObjectName("actionLockDevice")
        self.actionVoltageRangeAll = QtWidgets.QAction(MainWindow)
        self.actionVoltageRangeAll.setObjectName("actionVoltageRangeAll")
        self.actionResendData = QtWidgets.QAction(MainWindow)
        self.actionResendData.setObjectName("actionResendData")
        self.actionConvert_OASIS_Sample = QtWidgets.QAction(MainWindow)
        self.actionConvert_OASIS_Sample.setObjectName("actionConvert_OASIS_Sample")
        self.menuAbout.addAction(self.actionAbout)
        self.menuFile.addAction(self.actionConvert_OASIS_Sample)
        self.menuBar.addAction(self.menuFile.menuAction())
        self.menuBar.addAction(self.menuAbout.menuAction())

        self.retranslateUi(MainWindow)
        self.tabWidget.setCurrentIndex(0)
        self.pushButton.clicked.connect(self.actionSearch_Devices.trigger) # type: ignore
        self.checkBox_3.toggled['bool'].connect(self.label_18.setEnabled) # type: ignore
        self.checkBox_3.toggled['bool'].connect(self.lineEdit_3.setEnabled) # type: ignore
        self.pushButton_2.clicked.connect(self.actionSerial_Sample.trigger) # type: ignore
        self.comboBox.currentIndexChanged['QString'].connect(self.actionDevice_Selected_Changed.trigger) # type: ignore
        self.comboBox_2.currentIndexChanged['QString'].connect(self.actionRange_Channel1_Changed.trigger) # type: ignore
        self.pushButton_3.clicked.connect(self.actionshow_Previous_Data.trigger) # type: ignore
        self.pushButton_4.clicked.connect(self.actionsave_Previous_Data.trigger) # type: ignore
        self.checkBox_6.toggled['bool'].connect(self.radioButton.setEnabled) # type: ignore
        self.checkBox_6.toggled['bool'].connect(self.radioButton_2.setEnabled) # type: ignore
        self.label_21.linkActivated['QString'].connect(self.actionLockDevice.trigger) # type: ignore
        self.comboBox_11.currentIndexChanged['QString'].connect(self.actionVoltageRangeAll.trigger) # type: ignore
        self.pushButton_7.clicked.connect(self.actionResendData.trigger) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "OASIS Control Panel"))
        self.groupBox.setTitle(_translate("MainWindow", "Data Acquisition Configuration"))
        self.label_12.setText(_translate("MainWindow", "Sample Time (s):"))
        self.label_13.setText(_translate("MainWindow", "Sampling Frequency (Hz):"))
        self.lineEdit.setText(_translate("MainWindow", "2"))
        self.lineEdit_2.setText(_translate("MainWindow", "10000"))
        self.groupBox_2.setTitle(_translate("MainWindow", "Level Trigger"))
        self.checkBox_3.setText(_translate("MainWindow", "Enable Level Trigger"))
        self.label_18.setText(_translate("MainWindow", "Trigger Level (Channel 1):"))
        self.lineEdit_3.setText(_translate("MainWindow", "0.2"))
        self.groupBox_11.setTitle(_translate("MainWindow", "Voltage Ranges"))
        self.label_24.setText(_translate("MainWindow", "Ch. 7 (V):"))
        self.label_23.setText(_translate("MainWindow", "Ch. 6 (V):"))
        self.label_17.setText(_translate("MainWindow", "Ch. 4 (V):"))
        self.label_25.setText(_translate("MainWindow", "Ch. 5 (V):"))
        self.label_14.setText(_translate("MainWindow", "Ch. 1 (V):"))
        self.label_16.setText(_translate("MainWindow", "Ch. 3 (V):"))
        self.label_15.setText(_translate("MainWindow", "Ch. 2 (V):"))
        self.label_26.setText(_translate("MainWindow", "Ch. 8 (V):"))
        self.label_30.setText(_translate("MainWindow", "Set for all Channels (V):"))
        self.label_29.setText(_translate("MainWindow", "Oversampling Factor"))
        self.groupBox_3.setTitle(_translate("MainWindow", "Postprocessing Options"))
        self.checkBox_4.setText(_translate("MainWindow", "Show Data Plots after sampling"))
        self.checkBox_5.setText(_translate("MainWindow", "Save Data as .mat file"))
        self.pushButton_3.setText(_translate("MainWindow", "Show Previous Data"))
        self.pushButton_4.setText(_translate("MainWindow", "Save Previous Data"))
        self.pushButton_5.setText(_translate("MainWindow", "Resend Previous Data"))
        self.groupBox_4.setTitle(_translate("MainWindow", "OASIS Device Log"))
        self.progressBar.setFormat(_translate("MainWindow", "%p%"))
        self.groupBox_7.setTitle(_translate("MainWindow", "Sample Control"))
        self.pushButton_2.setText(_translate("MainWindow", "Start Data Acquisition"))
        self.pushButton_7.setText(_translate("MainWindow", "Resend Previous Data"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab), _translate("MainWindow", "Serial Data Acquisition"))
        self.label.setText(_translate("MainWindow", "<html><head/><body><p><span style=\" color:#0065bd;\">OASIS Control Panel</span></p></body></html>"))
        self.comboBox.setCurrentText(_translate("MainWindow", "Searching Devices..."))
        self.comboBox.setItemText(0, _translate("MainWindow", "Searching Devices..."))
        self.pushButton.setText(_translate("MainWindow", "Search Devices"))
        self.groupBox_5.setTitle(_translate("MainWindow", "Wireless Sample Synchronization"))
        self.checkBox_6.setText(_translate("MainWindow", "Enable Synchronization"))
        self.radioButton.setText(_translate("MainWindow", "WSS Source"))
        self.radioButton_2.setText(_translate("MainWindow", "WSS Sink"))
        self.label_20.setText(_translate("MainWindow", "Selected device is:"))
        self.groupBox_6.setTitle(_translate("MainWindow", "Selected Device Information"))
        self.checkBox.setText(_translate("MainWindow", "TEDS Readout"))
        self.label_6.setText(_translate("MainWindow", "Installed ADC:"))
        self.label_27.setText(_translate("MainWindow", "Board Architecture:"))
        self.label_4.setText(_translate("MainWindow", "Hardware Version:"))
        self.checkBox_2.setText(_translate("MainWindow", "Wireless Sample Synchronization"))
        self.label_7.setText(_translate("MainWindow", "Hardware Features:"))
        self.label_3.setText(_translate("MainWindow", "Board Name:"))
        self.label_5.setText(_translate("MainWindow", "Firmware Version:"))
        self.menuAbout.setTitle(_translate("MainWindow", "Help"))
        self.menuFile.setTitle(_translate("MainWindow", "File"))
        self.actionAbout.setText(_translate("MainWindow", "About"))
        self.actionSearch_Devices.setText(_translate("MainWindow", "Search Devices"))
        self.actionSerial_Sample.setText(_translate("MainWindow", "Serial_Sample"))
        self.actionDevice_Selected_Changed.setText(_translate("MainWindow", "Device_Selected_Changed"))
        self.actionRange_Channel1_Changed.setText(_translate("MainWindow", "Range_Channel1_Changed"))
        self.actionshow_Previous_Data.setText(_translate("MainWindow", "show_Previous_Data"))
        self.actionsave_Previous_Data.setText(_translate("MainWindow", "save_Previous_Data"))
        self.actionLockDevice.setText(_translate("MainWindow", "LockDevice"))
        self.actionLockDevice.setToolTip(_translate("MainWindow", "LockDevice"))
        self.actionVoltageRangeAll.setText(_translate("MainWindow", "VoltageRangeAll"))
        self.actionVoltageRangeAll.setToolTip(_translate("MainWindow", "VoltageRangeAll"))
        self.actionResendData.setText(_translate("MainWindow", "ResendData"))
        self.actionConvert_OASIS_Sample.setText(_translate("MainWindow", "Convert OASIS Sample"))
import UI_resources_rc



'''
 OASIS-Gui.py
 
	Main class of the OASIS-GUI used for interacting with OASIS devices

  Copyright (c) 2024 Oliver Zobel - MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"),
  to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
  IN THE SOFTWARE.
  
 '''

import sys
import serial
import time

from PyQt5 import QtWidgets, QtCore, QtGui
from PyQt5.QtGui import QMovie
from PyQt5.QtWidgets import QApplication, QDialog, QMainWindow
from PyQt5.QtCore import QObject, QThread, pyqtSignal
from PyQt5.uic import loadUi
from ui.OASISUI import Ui_MainWindow

from src.searchDevices import searchDevices
from src.sampleHandler import sampleHandler

QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_EnableHighDpiScaling, True)
QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_UseHighDpiPixmaps, True)
QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_DisableWindowContextHelpButton, True)

if sys.platform.startswith('win'):
    import ctypes
    myappid = 'OASIS-GUI'
    ctypes.windll.shell32.SetCurrentProcessExplicitAppUserModelID(myappid)

DeviceSearch = searchDevices()
SampleHandler = sampleHandler()

class WorkerDeviceSearch(QObject):
    
    finished = pyqtSignal()
    printLogSignal = pyqtSignal(str)
    
    def __init__(self, DeviceSearch):
        super(WorkerDeviceSearch, self).__init__()
        self.DeviceSearch = DeviceSearch

    def run(self):
        self.DeviceSearch.SerialSearch(self.printLogSignal)
        self.finished.emit()
        
class WorkerSerialSample(QObject):
    
    finished = pyqtSignal()
    printLogSignal = pyqtSignal(str)
    sampleAborted = pyqtSignal()
    sampleProgress = pyqtSignal(int)
    
    def __init__(self, SampleHandler):
        super(WorkerSerialSample, self).__init__()
        self.SampleHandler = SampleHandler

    def newSample(self):
        self.SampleHandler.SampleSerial(self.printLogSignal, self.sampleAborted, self.sampleProgress)
        self.finished.emit()
        
    def resendPreviousSample(self):
        self.SampleHandler.resendSampleSerial(self.printLogSignal, self.sampleAborted, self.sampleProgress)
        self.finished.emit()

class Window(QMainWindow, Ui_MainWindow):

    def __init__(self, parent=None):
        super().__init__(parent)
        self.setupUi(self)
        self.connectSignalsSlots()
        self.progressBar_2.setVisible(False)
        self.label_19.setVisible(False)
        self.movie = QMovie(":/Misc/resources/spin.gif")
        self.label_19.setMovie(self.movie)
        self.movie.start()
        self.DeviceLocked = False
        self.RelockDevice = False
        self.Search_Devices()

    def connectSignalsSlots(self):
        self.actionAbout.triggered.connect(self.About)
        self.actionSearch_Devices.triggered.connect(self.Search_Devices)
        self.actionSerial_Sample.triggered.connect(self.Serial_Sample)
        self.actionResendData.triggered.connect(self.Resend_Sample)
        self.actionDevice_Selected_Changed.triggered.connect(self.Update_Device)
        self.actionRange_Channel1_Changed.triggered.connect(self.Update_Range)
        self.actionshow_Previous_Data.triggered.connect(self.Show_Previous_Data)
        self.actionsave_Previous_Data.triggered.connect(self.Save_Previous_Data)
        self.label_21.mousePressEvent = self.LockDevice
        self.actionVoltageRangeAll.triggered.connect(self.Update_VoltageRangesAll)

    def About(self):
        dialog = AboutDialog(self)
        dialog.exec()
        
    def Update_Device(self):
        DeviceSearch.UpdateSelectedDevice(self)
    
    def Update_VoltageRangesAll(self):
        DeviceSearch.UpdateAllVoltageRanges(self)
        
    def Update_Range(self):
        if DeviceSearch.Devices and len(DeviceSearch.Devices[self.comboBox.currentIndex()][1])==7:
            if(DeviceSearch.Devices[self.comboBox.currentIndex()][1][2]=="16"):
                self.comboBox_3.setCurrentIndex(self.comboBox_2.currentIndex())
                self.comboBox_4.setCurrentIndex(self.comboBox_2.currentIndex())
                self.comboBox_5.setCurrentIndex(self.comboBox_2.currentIndex())
        
    def Search_Devices(self):
        # Lock & Update GUI
        self.pushButton.setEnabled(False)
        self.tabWidget.setEnabled(False)
        self.comboBox.setEnabled(False)
        self.pushButton_3.setEnabled(False)
        self.pushButton_4.setEnabled(False)
        self.pushButton_7.setEnabled(False)
        self.textEdit.clear()
        self.label_19.setVisible(True)
        DeviceSearch.Devices = []
        DeviceSearch.UpdateDeviceList(self)
        DeviceSearch.UpdateSelectedDevice(self)
        self.comboBox.setItemText(0, "Searching Devices...")
        
        # Setup Worker & Thread
        self.thread = QThread()
        self.workerDeviceSearch = WorkerDeviceSearch(DeviceSearch)
        self.workerDeviceSearch.moveToThread(self.thread)
        
        # Connect Signals
        self.workerDeviceSearch.finished.connect(self.thread.quit)
        self.workerDeviceSearch.printLogSignal.connect(self.printLog)
        self.thread.started.connect(self.workerDeviceSearch.run)
        self.thread.finished.connect(self.Search_Devices_PostProcess)
        
        self.thread.start()
        
    def Search_Devices_PostProcess(self):
        self.pushButton.setEnabled(True)
        DeviceSearch.UpdateDeviceList(self)
        DeviceSearch.UpdateSelectedDevice(self)
        self.label_19.setVisible(False)
        
    def printLog(self, string):
        self.textEdit.insertPlainText(string)
        self.textEdit.ensureCursorVisible()
        
    def Serial_Sample(self):
        # Lock & Update GUI
        self.tabWidget.setEnabled(False)
        self.pushButton.setEnabled(False)
        self.pushButton_2.setEnabled(False)
        self.comboBox.setEnabled(False)
        self.progressBar.setValue(0)
        self.progressBar_2.setVisible(False)
        self.groupBox_5.setEnabled(False)
        self.sampleError = False
        self.LastSampleDevice = DeviceSearch.Devices[self.comboBox.currentIndex()][1][5]

        # Release Serial lock
        if self.DeviceLocked:
            try:
                self.LockSerial.close()
                time.sleep(0.5)
            except:
                pass
            self.RelockDevice = True
            self.DeviceLocked = False

        # Get Acquisition Paramters from user input
        SampleHandler.getAcquisitionParameters(self, DeviceSearch.Devices[self.comboBox.currentIndex()])
        
        # Setup Worker & Thread
        self.thread = QThread()
        self.workerSerialSample = WorkerSerialSample(SampleHandler)
        self.workerSerialSample.moveToThread(self.thread)
        
        # Connect Signals
        self.workerSerialSample.finished.connect(self.thread.quit)
        self.workerSerialSample.printLogSignal.connect(self.printLog)
        self.workerSerialSample.sampleAborted.connect(self.abortSample)
        self.workerSerialSample.sampleProgress.connect(self.updateProgressBar)
        self.thread.started.connect(self.workerSerialSample.newSample)
        self.thread.finished.connect(self.Serial_Sample_PostProcess)
        
        self.thread.start()
        
    def Resend_Sample(self):
        # Lock & Update GUI
        self.tabWidget.setEnabled(False)
        self.pushButton.setEnabled(False)
        self.pushButton_2.setEnabled(False)
        self.comboBox.setEnabled(False)
        self.progressBar.setValue(0)
        self.progressBar_2.setVisible(False)
        self.groupBox_5.setEnabled(False)
        self.sampleError = False
        self.LastSampleDevice = DeviceSearch.Devices[self.comboBox.currentIndex()][1][5]

        # Release Serial lock
        if self.DeviceLocked:
            try:
                self.LockSerial.close()
                time.sleep(0.5)
            except:
                pass
            self.RelockDevice = True
            self.DeviceLocked = False
        
        # Setup Worker & Thread
        self.thread = QThread()
        self.workerSerialSample = WorkerSerialSample(SampleHandler)
        self.workerSerialSample.moveToThread(self.thread)
        
        # Connect Signals
        self.workerSerialSample.finished.connect(self.thread.quit)
        self.workerSerialSample.printLogSignal.connect(self.printLog)
        self.workerSerialSample.sampleAborted.connect(self.abortSample)
        self.workerSerialSample.sampleProgress.connect(self.updateProgressBar)
        self.thread.started.connect(self.workerSerialSample.resendPreviousSample)
        self.thread.finished.connect(self.Serial_Sample_PostProcess)
        
        self.thread.start()

    def abortSample(self):
        self.sampleError = True
        self.progressBar_2.setVisible(True)
        self.progressBar.setValue(0)
        self.pushButton.setEnabled(True)
        self.pushButton_2.setEnabled(True)
        self.comboBox.setEnabled(True)
        self.tabWidget.setEnabled(True)

    def updateProgressBar(self, value):
        self.progressBar.setValue(value)
        
    def Serial_Sample_PostProcess(self):
        if self.checkBox_4.isChecked() and not self.sampleError:
            self.Show_Previous_Data()
        if self.checkBox_5.isChecked() and not self.sampleError:
            self.Save_Previous_Data()
        self.pushButton.setEnabled(True)
        self.pushButton_2.setEnabled(True)
        self.pushButton_3.setEnabled(True)
        self.pushButton_4.setEnabled(True)
        self.pushButton_7.setEnabled(True)
        if DeviceSearch.Devices[self.comboBox.currentIndex()][1][4]:
            self.groupBox_5.setEnabled(True)
        if self.RelockDevice:
            self.LockDevice("JA")
        if not self.DeviceLocked:
            self.comboBox.setEnabled(True)
        self.tabWidget.setEnabled(True)
        
    def Show_Previous_Data(self):
        SampleHandler.plotData()
    
    def Save_Previous_Data(self):
        SampleHandler.saveData(self)

    def LockDevice(self, garbage):
        self.DeviceLocked = not self.DeviceLocked
        if self.DeviceLocked:
            self.textEdit.append(f"[OASIS-GUI]: Locking {self.comboBox.currentText()}\n")
            try:
                self.LockSerial = serial.Serial(port=DeviceSearch.Devices[self.comboBox.currentIndex()][0], baudrate=DeviceSearch.serialSpeed, timeout=2)
            except (OSError, serial.SerialException):
                self.textEdit.append("[OASIS-GUI]: DEVICE ERROR! Could not lock device.\n")
                self.DeviceLocked = False

            self.comboBox.setEnabled(False)
            self.label_21.setPixmap(QtGui.QPixmap(":/Misc/resources/lock.png"))
        else:
            try:
                self.LockSerial.close()
            except:
                pass

        if not self.DeviceLocked:
            self.textEdit.append(f"[OASIS-GUI]: Unlocking {self.comboBox.currentText()}\n")
            self.comboBox.setEnabled(True)
            self.label_21.setPixmap(QtGui.QPixmap(":/Misc/resources/unlock.png"))

class AboutDialog(QDialog):
    def __init__(self, parent=None):
        super().__init__(parent)
        loadUi("ui/OASISGui_About.ui", self)

app = QApplication(sys.argv)

# set app icon    
app_icon = QtGui.QIcon()
app_icon.addFile(':/Icons/resources/icons/16x16.png', QtCore.QSize(16,16))
app_icon.addFile(':/Icons/resources/icons/24x24.png', QtCore.QSize(24,24))
app_icon.addFile(':/Icons/resources/icons/32x32.png', QtCore.QSize(32,32))
app_icon.addFile(':/Icons/resources/icons/48x48.png', QtCore.QSize(48,48))
app_icon.addFile(':/Icons/resources/icons/64x64.png', QtCore.QSize(64,64))
app_icon.addFile(':/Icons/resources/icons/256x256.png', QtCore.QSize(256,256))
app.setWindowIcon(app_icon)
win = Window()
win.show()
sys.exit(app.exec())


'''
 searchDevices.py
 
	Class handling search of OASIS devices

  Copyright (c) 2024 Oliver Zobel - MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"),
  to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
  IN THE SOFTWARE.
  
 '''

import serial
import serial.tools.list_ports
import time

from PyQt5 import QtGui

class searchDevices():
    def __init__(self):
        
        self.Devices = []
        
        return
        
    def SerialSearch(self, printLogSignal):
        
        self.Devices = []
        
        printLogSignal.emit("[OASIS-GUI]: Searching devices...\n")
        
        self.serialSpeed = 1000000
    
        # Search all available COM devices
        comlist = serial.tools.list_ports.comports()
        connected = []
        for element in comlist:
            connected.append(element.device)
        
        if connected:
            
            for DeviceNum in range(0, len(connected)):
                printLogSignal.emit("[OASIS-GUI]: Found device on port " + connected[DeviceNum] + "\n")
                
                # Connect to Serial Device
                try:
                    s = serial.Serial(port=connected[DeviceNum], baudrate=self.serialSpeed, timeout=2)
                except (OSError, serial.SerialException):
                    printLogSignal.emit("[OASIS-GUI]: DEVICE ERROR! Could not open serial communication on port " + connected[DeviceNum] + "\n\n")
                    continue
                
                while s.inWaiting():
                    SerialAnswer = s.readline()
                    if SerialAnswer !=bytes("","utf-8"):
                        if SerialAnswer.startswith(bytes("[OASIS]","utf-8")):
                            printLogSignal.emit(SerialAnswer.decode("utf-8",errors="ignore"))
                
                # Get Device Information
                s.write(bytes("OASIS.RawInfo()","utf-8"))
                printLogSignal.emit("[OASIS-GUI]: Getting device info...\n")
                DeviceInfo = s.readline().decode("utf-8",errors="ignore").split(";")

                s.close
                if(len(DeviceInfo)==8):
                    isOASIS = True
                    printLogSignal.emit("[OASIS-GUI]: Device on port " + connected[DeviceNum] + " is an OASIS board\n")
                else:
                    isOASIS = False
                    DeviceInfo = [""]
                    printLogSignal.emit("[OASIS-GUI]: Device on port " + connected[DeviceNum] + " is unknown or does not respond\n")
                
                self.Devices.append([connected[DeviceNum],DeviceInfo,isOASIS,self.serialSpeed])
            
        else:
            printLogSignal.emit("[OASIS-GUI]: Did not find any devices\n")
            
        return
    
    def UpdateDeviceList(self, Window):
        
        # Clear Device List Combobox
        Window.comboBox.clear()
        
        # No Devices found
        if(len(self.Devices)==0):
            
            Window.comboBox.setEnabled(False)
            Window.comboBox.addItem("No Devices")
            
        else:
            
            Window.comboBox.setEnabled(True)
            
            # Sort Devices (OASIS first)
            DevicesOASIS = []
            DevicesOther = []
            
            for Device in range(0,len(self.Devices)):
                if(self.Devices[Device][2]):
                    DevicesOASIS.append(self.Devices[Device])
                else:
                    DevicesOther.append(self.Devices[Device])
            
            Window.textEdit.append("[OASIS-GUI]: Found " + str(len(DevicesOASIS)) + " OASIS Board(s) and " + str(len(DevicesOther)) + " unknown device(s).\n")
            Window.textEdit.repaint()
            
            self.Devices = DevicesOASIS
            for i in range(0,len(DevicesOther)):
                self.Devices.append(DevicesOther[i])
            
            for Device in range(0,len(self.Devices)):
                
                # Device found is an OASIS
                if(self.Devices[Device][2]):
                    Window.comboBox.addItem(self.Devices[Device][1][6] + " V." + self.Devices[Device][1][0] + " - " + self.Devices[Device][1][5] + " (" + self.Devices[Device][0] + ")")
                else:
                    Window.comboBox.addItem("Unknown Device (" + self.Devices[Device][0] + ")")
            
            Window.comboBox.repaint()
            
        return
    
    def UpdateSelectedDevice(self, Window):
        
        DeviceSelected = Window.comboBox.currentIndex()
        
        Window.comboBox_2.clear()
        Window.comboBox_3.clear()
        Window.comboBox_4.clear()
        Window.comboBox_5.clear()
        Window.comboBox_6.clear()
        Window.comboBox_7.clear()
        Window.comboBox_8.clear()
        Window.comboBox_9.clear()
        Window.comboBox_10.clear()
        Window.comboBox_11.clear()
        
        # No Device found
        if(len(self.Devices)==0):
            Window.label_2.setPixmap(QtGui.QPixmap(":/Boards/resources/boards/Unknown.png"))
            Window.label_8.setText("")
            Window.label_9.setText("")
            Window.label_10.setText("")
            Window.label_11.setText("")
            Window.label_28.setText("")
            Window.checkBox.setChecked(False)
            Window.checkBox_2.setChecked(False)
            
            # Disable sampling
            Window.tabWidget.setEnabled(False)
            Window.pushButton_2.setEnabled(False)
            
        else:
            if(self.Devices[DeviceSelected][2]):
                
                # Update GUI Text
                Window.label_28.setText(self.Devices[DeviceSelected][1][6]) # Device architecture
                Window.label_8.setText(self.Devices[DeviceSelected][1][5]) # Device Name
                Window.label_9.setText(self.Devices[DeviceSelected][1][0]) # Hardware Version
                Window.label_10.setText(self.Devices[DeviceSelected][1][1]) # Firmware Version
                
                # Set available Voltage ranges
                if(self.Devices[DeviceSelected][1][2]=="18"):
                    Window.label_11.setText("AD7606C-18 (18 Bit resolution)")
                    Window.comboBox_11.setEnabled(True)
                    Window.comboBox_11.addItem("2.5")
                    Window.comboBox_11.addItem("5.0")
                    Window.comboBox_11.addItem("6.25")
                    Window.comboBox_11.addItem("10")
                    Window.comboBox_11.addItem("12.5")
                    Window.comboBox_2.setEnabled(True)
                    Window.comboBox_2.addItem("2.5")
                    Window.comboBox_2.addItem("5.0")
                    Window.comboBox_2.addItem("6.25")
                    Window.comboBox_2.addItem("10")
                    Window.comboBox_2.addItem("12.5")
                    Window.comboBox_3.setEnabled(True)
                    Window.comboBox_3.addItem("2.5")
                    Window.comboBox_3.addItem("5.0")
                    Window.comboBox_3.addItem("6.25")
                    Window.comboBox_3.addItem("10")
                    Window.comboBox_3.addItem("12.5")
                    Window.comboBox_4.setEnabled(True)
                    Window.comboBox_4.addItem("2.5")
                    Window.comboBox_4.addItem("5.0")
                    Window.comboBox_4.addItem("6.25")
                    Window.comboBox_4.addItem("10")
                    Window.comboBox_4.addItem("12.5")
                    Window.comboBox_5.setEnabled(True)
                    Window.comboBox_5.addItem("2.5")
                    Window.comboBox_5.addItem("5.0")
                    Window.comboBox_5.addItem("6.25")
                    Window.comboBox_5.addItem("10")
                    Window.comboBox_5.addItem("12.5")
                    Window.comboBox_6.setEnabled(True)
                    Window.comboBox_6.addItem("2.5")
                    Window.comboBox_6.addItem("5.0")
                    Window.comboBox_6.addItem("6.25")
                    Window.comboBox_6.addItem("10")
                    Window.comboBox_6.addItem("12.5")
                    Window.comboBox_7.setEnabled(True)
                    Window.comboBox_7.addItem("2.5")
                    Window.comboBox_7.addItem("5.0")
                    Window.comboBox_7.addItem("6.25")
                    Window.comboBox_7.addItem("10")
                    Window.comboBox_7.addItem("12.5")
                    Window.comboBox_8.setEnabled(True)
                    Window.comboBox_8.addItem("2.5")
                    Window.comboBox_8.addItem("5.0")
                    Window.comboBox_8.addItem("6.25")
                    Window.comboBox_8.addItem("10")
                    Window.comboBox_8.addItem("12.5")
                    Window.comboBox_9.setEnabled(True)
                    Window.comboBox_9.addItem("2.5")
                    Window.comboBox_9.addItem("5.0")
                    Window.comboBox_9.addItem("6.25")
                    Window.comboBox_9.addItem("10")
                    Window.comboBox_9.addItem("12.5")
                    Window.comboBox_10.setEnabled(True)
                    Window.comboBox_10.addItem("x1")
                    Window.comboBox_10.addItem("x2")
                    Window.comboBox_10.addItem("x4")
                    Window.comboBox_10.addItem("x8")
                    Window.comboBox_10.addItem("x16")
                    Window.comboBox_10.addItem("x32")
                    Window.comboBox_10.addItem("x64")
                    Window.comboBox_10.addItem("x128")
                    Window.comboBox_10.addItem("x256")
                    Window.comboBox_10.setCurrentIndex(2)
                elif(self.Devices[DeviceSelected][1][2]=="16"):
                    Window.label_11.setText("AD7606-4 (16 Bit resolution)")
                    Window.comboBox_2.setEnabled(True)
                    Window.comboBox_3.setEnabled(False)
                    Window.comboBox_4.setEnabled(False)
                    Window.comboBox_5.setEnabled(False)
                    Window.comboBox_2.addItem("5.0")
                    Window.comboBox_2.addItem("10.0")
                    Window.comboBox_3.addItem("5.0")
                    Window.comboBox_3.addItem("10.0")
                    Window.comboBox_4.addItem("5.0")
                    Window.comboBox_4.addItem("10.0")
                    Window.comboBox_5.addItem("5.0")
                    Window.comboBox_5.addItem("10.0")
                else:
                    Window.label_11.setText("Unknown ADC")
                
                # Detect Hardware Features
                if(self.Devices[DeviceSelected][1][3]=="1"):
                    Window.checkBox.setChecked(True)
                else:
                    Window.checkBox.setChecked(False)
                    
                if(self.Devices[DeviceSelected][1][4]=="1"):
                    Window.checkBox_2.setChecked(True)
                    Window.checkBox_6.setEnabled(True)
                    Window.groupBox_5.setEnabled(True)
                    Window.label_21.setEnabled(True)
                else:
                    Window.checkBox_2.setChecked(False)
                    Window.groupBox_5.setEnabled(False)
                    Window.checkBox_6.setEnabled(False)
                    Window.label_21.setEnabled(False)
                
                # Set Board Image
                if(self.Devices[DeviceSelected][1][6]=="Original OASIS"):
                    Window.label_2.setPixmap(QtGui.QPixmap(":/Boards/resources/boards/OASISV1.png"))
                elif(self.Devices[DeviceSelected][1][6]=="OASIS-UROS"):
                    Window.label_2.setPixmap(QtGui.QPixmap(":/Boards/resources/boards/OASIS-UROS.png"))
        
                # Enable sampling
                Window.tabWidget.setEnabled(True)
                Window.pushButton_2.setEnabled(True)
                
            else:
                Window.label_2.setPixmap(QtGui.QPixmap(":/Boards/resources/boards/Unknown.png"))
                Window.label_28.setText("Unknown")
                Window.label_8.setText("Unknown")
                Window.label_9.setText("Unknown")
                Window.label_10.setText("Unknown")
                Window.label_11.setText("Unknown")
                Window.checkBox.setChecked(False)
                Window.checkBox_2.setChecked(False)
                
                # Disable sampling
                Window.tabWidget.setEnabled(False)
                Window.pushButton_2.setEnabled(False)
                
        return
    
    def UpdateAllVoltageRanges(self, window):
        
        window.comboBox_2.setCurrentIndex(window.comboBox_11.currentIndex())
        window.comboBox_3.setCurrentIndex(window.comboBox_11.currentIndex())
        window.comboBox_4.setCurrentIndex(window.comboBox_11.currentIndex())
        window.comboBox_5.setCurrentIndex(window.comboBox_11.currentIndex())
        window.comboBox_6.setCurrentIndex(window.comboBox_11.currentIndex())
        window.comboBox_7.setCurrentIndex(window.comboBox_11.currentIndex())
        window.comboBox_8.setCurrentIndex(window.comboBox_11.currentIndex())
        window.comboBox_9.setCurrentIndex(window.comboBox_11.currentIndex())
        
        return